﻿using System;
using Xamarin.Forms.Platform.iOS;
using Xamarin.Forms;
using UIKit;
using CoreGraphics;
using FormsComponentDemo;
using FormsComponentDemo.iOS;

[assembly: ExportRenderer(typeof(FormEntry), typeof(FormEntryRenderer))]

namespace FormsComponentDemo.iOS
{
	public class FormEntryRenderer : ViewRenderer<FormEntry, UIFormEntry>
	{
		protected override void OnElementChanged (ElementChangedEventArgs<FormEntry> e)
		{
			base.OnElementChanged (e);

			if (e.OldElement != null)
			{
				return;
			}

			if (Control == null)
			{
				//BackgroundColor = Color.Yellow.ToUIColor();

				// create and add the native control to the forms control
				UIFormEntry native = new UIFormEntry (Bounds);

				SetNativeControl(native);

				Control.TextChanged += HandleTextChanged;
			}

			SetProperties (null);
		}

		void HandleTextChanged (string obj)
		{
			Console.WriteLine("1 - UITextField - HandleTextChanged called");
			//			e.NewElement.SendMyEvent();

			// send the text value back to the forms control
			Element.Text = obj;
		}

		protected override void OnElementPropertyChanged (object sender, System.ComponentModel.PropertyChangedEventArgs e)
		{
			base.OnElementPropertyChanged (sender, e);

			Console.WriteLine ("OnElementPropertyChanged called");
			SetProperties (sender);
		}

		private void SetProperties(object sender)
		{
			var native = Control;
			native.Text = Element.Text;
			native.Validated = Element.Validated;

			if (sender != null)
			{
				FormEntry s = (FormEntry)sender;
				s.SendControlChanged ();
			}
		}
	}


	// CONTROL UI CODE

	public class UIFormEntry : UIView
	{
		public string Text {

			get {
				return txtText.Text;
			}

			set {
				txtText.Text = value;

				if(TextChanged != null)
					TextChanged (value);
			}
		}

		private bool validated = false;
		public bool Validated {

			set {
				Console.WriteLine ("Setting property Validated");
				validated = value;
				if (value == true)
				{
					UIView.Animate (0.25f, delegate {
						imgCancel.Alpha = 0f;
						imgOk.Alpha = 1f;
					});
				}
				else
				{
					UIView.Animate (0.25f, delegate {
						imgCancel.Alpha = 1f;
						imgOk.Alpha = 0f;
					});
				}
			}

			get {
				return validated;
			}
		}



		UIView leftView;
		UITextField txtText;
		UIImageView imgCancel, imgOk;

		public UIFormEntry(CoreGraphics.CGRect bounds) : base(bounds)
		{
			BackgroundColor = UIColor.Red;

			// we can only set the height here
			Frame = new CGRect(0, 0, 0, 50);


			leftView = new UIView (new CGRect (0, 0, Frame.Height, Frame.Height));
			leftView.BackgroundColor = Global.ColorLight.ToUIColor();
			AddSubview (leftView);

			BackgroundColor = Global.ColorDark.ToUIColor().ColorWithAlpha(0.25f);
			txtText = new UITextField (new CGRect (Frame.Height + 5, 5, Frame.Width - 10 - Frame.Height, Frame.Height - 10));
			txtText.Font = UIFont.FromName (Global.FontName, 18);
			AddSubview (txtText);

			txtText.EditingChanged += TxtText_ValueChanged;

			// setup the icons
			imgCancel = new UIImageView(UIImage.FromFile("cancel.png"));
			CGRect f = imgCancel.Frame;
			f.X = (leftView.Frame.Width - f.Width)/2;
			f.Y = (leftView.Frame.Height - f.Height)/2;
			imgCancel.Frame = f;

			leftView.AddSubview (imgCancel);

			imgOk = new UIImageView (UIImage.FromFile ("ok.png"));
			f = imgOk.Frame;
			f.X = (leftView.Frame.Width - f.Width)/2;
			f.Y = (leftView.Frame.Height - f.Height)/2;
			imgOk.Frame = f;

			leftView.AddSubview (imgOk);

			imgOk.Alpha = 0f;
		}

		public override void LayoutSubviews ()
		{
			base.LayoutSubviews ();

			// layout the controls
			CGRect f = Frame;

			leftView.Frame = new CGRect (0, 0, Frame.Height, Frame.Height);
			txtText.Frame = new CGRect (Frame.Height + 5, 5, Frame.Width - 10 - Frame.Height, Frame.Height - 10);

			// center the icons in the "leftview" frame
			CGRect cf = imgCancel.Frame;
			cf.X = (leftView.Frame.Width - cf.Width)/2;
			cf.Y = (leftView.Frame.Height - cf.Height)/2;
			imgCancel.Frame = cf;

			cf = imgOk.Frame;
			cf.X = (leftView.Frame.Width - cf.Width)/2;
			cf.Y = (leftView.Frame.Height - cf.Height)/2;
			imgOk.Frame = cf;
		}

		public event Action<string> TextChanged;
		void TxtText_ValueChanged (object sender, EventArgs e)
		{
			if (TextChanged != null)
			{
				TextChanged (((UITextField)sender).Text);
				//Console.WriteLine ("UIFormEntry - TextChanged event");
			}
		}


		public override UIColor BackgroundColor {
			get {
				return base.BackgroundColor;
			}
			set {
				base.BackgroundColor = value;

				if(txtText != null)
					txtText.BackgroundColor = value;
			}
		}

		public string Placeholder
		{
			set {
				txtText.Placeholder = value;
			}
		}

		public override bool IsFirstResponder {
			get {
				return txtText.IsFirstResponder;
			}
		}

		public override bool ResignFirstResponder ()
		{
			return txtText.ResignFirstResponder ();
		}

	}
}

